// --------------------------------------------------------------------------
//
// Media Feed Screen Saver
// http://blog.debin.net/Media-feed-screen-saver/
// damien@debin.net
//
// --------------------------------------------------------------------------
//
// Copyright (c) 2006 Damien Debin.
// All rights reserved.
// 
// This file is part of Media Feed Screen Saver.
// 
// Media Feed Screen Saver is free software; you can redistribute it
// and/or modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
// 
// Foobar is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Foobar; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
// --------------------------------------------------------------------------

using System;
using System.Windows.Forms;
using System.Drawing;
using ST = System.Timers;

namespace MediaFeedScreenSaver
{
    /// <summary>
    /// Encapsulates the rendering of the description of an item.
    /// </summary>
    /// <typeparam name="T">The type of item that this ItemDescriptionView will draw.</typeparam>
    public class ItemDescriptionView
    {
        private Point location;
        private Size size;

        private Color foreColor;
        private Font titleFont;
        private Rss.RssItem displayItem;
        private bool allPicturesStillNotDownloaded = false;

        // The initial alpha value and the amount to change the value by each time
        private int textAlpha = 0;
        private int textAlphaCounter = 0;
        private int textAlphaCounterStep;
        private readonly int textAlphaCounterDelta = 30;
        private readonly int textAlphaMax = 255;
        private readonly int textAlphaCounterMax;
        public readonly int fadeTimerInterval = 150;

        public Rss.RssItem DisplayItem { get { return displayItem; } set { displayItem = value; } }
        public Point Location { get { return location; } set { location = value; } }
        public Size Size { get { return size; } set { size = value; } }
        public Color ForeColor { get { return foreColor; } set { foreColor = value; } }
        public Font TitleFont { get { return titleFont; } set { titleFont = value; } }
        public int TextAlpha { get { return textAlpha; } }
        public bool AllPicturesStillNotDownloaded { get { return allPicturesStillNotDownloaded; } set { allPicturesStillNotDownloaded = value; } }

        /// <summary>
        /// Create a new ItemDescriptionView connected to <paramref name="listView"/>.
        /// </summary>
        /// <param name="listView"></param>
        public ItemDescriptionView()
        {
            if (!Properties.Settings.Default.TitleFading)
            {
                fadeTimerInterval = Properties.Settings.Default.MsecBetweenPictures;
            }

            textAlphaCounterMax = (Properties.Settings.Default.MsecBetweenPictures / (fadeTimerInterval * 2)) * textAlphaCounterDelta;
            textAlphaCounterStep = textAlphaCounterDelta;
        }

        public void Paint(PaintEventArgs e)
        {
            Rectangle bgRect, textRect;

            if (!Properties.Settings.Default.TitleFading)
            {
                textAlpha = textAlphaMax;
            }

            // Change the graphics settings to draw clear text
            e.Graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;
            e.Graphics.TextRenderingHint = System.Drawing.Text.TextRenderingHint.ClearTypeGridFit;

            string textToDraw = String.Empty;

            if (((Properties.Settings.Default.ShowTitles & 1) != 0) && (displayItem != null))
            {
                textToDraw = displayItem.Title;
            }

            if ((Properties.Settings.Default.ShowTitles & 2) != 0)
            {
                if (displayItem == null) textToDraw = "[ downloading feed... ]";
                else if (String.IsNullOrEmpty(displayItem.EnclosureImageURI)) textToDraw = "[ no media enclosure found ] " + textToDraw;
                else if (!displayItem.Enable) textToDraw = "[ media not yet downloaded ] " + textToDraw;
                else if (allPicturesStillNotDownloaded) textToDraw = "[ still some pictures to download... ] " + textToDraw;
            }

            if (String.IsNullOrEmpty(textToDraw))
            {
                return;
            }

            using (StringFormat textFormat = new StringFormat(StringFormatFlags.LineLimit))
            {
                SizeF sf = e.Graphics.MeasureString(textToDraw, titleFont, Size.Width, textFormat);
                sf.Width *= 1.02f;
                //sf.Height *= 1.01f;
                int rWidth = Math.Min((int)sf.Width, size.Width);
                int rHeight = Math.Min((int)sf.Height, size.Height);
                int margin = (int)(sf.Height / 4);
                textRect = new Rectangle(Location.X - rWidth - margin, Location.Y - rHeight - margin, rWidth, rHeight);
                bgRect = textRect;
                bgRect.Inflate(margin, margin);

                // Draw the alpha box
                using (Brush bgBrush = new SolidBrush(Color.FromArgb(textAlpha / 4, 0, 0, 0)))
                {
                    e.Graphics.FillRectangle(bgBrush, bgRect);
                }

                // Draw the title of the article
                textFormat.Alignment = StringAlignment.Near;
                textFormat.LineAlignment = StringAlignment.Near;
                textFormat.Trimming = StringTrimming.EllipsisWord;
                using (Brush textBrush = new SolidBrush(Color.FromArgb(textAlpha, ForeColor)))
                {
                    e.Graphics.DrawString(textToDraw, titleFont, textBrush, textRect, textFormat);
                }
            }
        }

        public bool titleFadingAnimate()
        {
            bool titleAnimationEnd = false;

            if (!Properties.Settings.Default.TitleFading)
            {
                return true;
            }

            // Change the alpha value of the text being drawn
            // Moves up until it reaches textAlphaMax and then moves down
            textAlphaCounter += textAlphaCounterStep;
            if (textAlphaCounter >= textAlphaCounterMax)
            {
                textAlphaCounter = textAlphaCounterMax;
                textAlphaCounterStep *= -1;
            }
            else if (textAlphaCounter <= 0)
            {
                titleAnimationEnd = true;
                textAlphaCounter = 0;
                textAlphaCounterStep *= -1;
            }
            textAlpha = Math.Min(textAlphaMax, textAlphaCounter);

            return titleAnimationEnd;
        }
    }
}
