// --------------------------------------------------------------------------
//
// Media Feed Screen Saver
// http://blog.debin.net/Media-feed-screen-saver/
// damien@debin.net
//
// --------------------------------------------------------------------------
//
// Copyright (c) 2006 Damien Debin.
// All rights reserved.
// 
// This file is part of Media Feed Screen Saver.
// 
// Media Feed Screen Saver is free software; you can redistribute it
// and/or modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
// 
// Foobar is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Foobar; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
// --------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Net;
using System.Net.Cache;
using System.Runtime.CompilerServices;
using System.Threading;
using System.Xml;
using System.Drawing;

namespace MediaFeedScreenSaver.Rss
{
    /// <summary>
    /// Representation of an RSS element in a RSS 2.0 XML document
    /// </summary>
    class RssFeed : IDisposable
    {
        private List<RssChannel> channels;
        public IList<RssChannel> Channels { get { return channels.AsReadOnly(); } }
        public RssChannel MainChannel { get { if ((channels == null) || (channels.Count == 0)) return null; else return Channels[0]; } }

        private readonly Size imageSize;
        private string feedURI;

        /// <summary>
        /// Private constructor to be used with factory pattern.  
        /// </summary>
        /// <param name="xmlNode">An XML block containing the RSSFeed content.</param>
        public RssFeed(XmlNode xmlNode, string uri, Size requestedImageSize)
        {
            imageSize = requestedImageSize;
            if (xmlNode != null)
            {
                LoadFromNode(xmlNode);
            }
            feedURI = uri;
        }

        /// <summary>
        /// Factory that constructs RSSFeed objects from a uri pointing to a valid RSS 2.0 XML file.
        /// </summary>
        /// <exception cref="System.Net.WebException">Occurs when the uri cannot be located on the web.</exception>
        /// <param name="uri">The URL to read the RSS feed from.</param>
        public static RssFeed FromUri(string uri, Size requestedImageSize)
        {
            RssFeed feed = null;

            feed = new RssFeed(null, uri, requestedImageSize);
            feed.RefreshFeed();

            return feed;
        }

        [MethodImpl(MethodImplOptions.Synchronized)]
        private void LoadFromNode(XmlNode xmlNode)
        {
            DisposeChannels();

            channels = new List<RssChannel>();

            // Read the <rss> tag
            XmlNode rssNode = xmlNode.SelectSingleNode("rss");

            // For each <channel> node in the <rss> node
            // add a channel.
            XmlNodeList channelNodes = rssNode.ChildNodes;
            foreach (XmlNode channelNode in channelNodes)
            {
                RssChannel newChannel = new RssChannel(channelNode, imageSize);
                channels.Add(newChannel);
            }
        }

        public void RefreshFeed()
        {
            if (feedURI == null) return;

            lock (RssItem.concurrentDownloadLock) //to prevent multiple concurrent downloads
            {
                try
                {
                    WebClient webClient = new WebClient();
                    webClient.CachePolicy = new RequestCachePolicy(RequestCacheLevel.Default);
                    using (Stream webStream = webClient.OpenRead(feedURI))
                    {
                        XmlDocument xmlDoc = new XmlDocument();
                        xmlDoc.Load(webStream);
                        LoadFromNode(xmlDoc);
                    }
                }
                catch (WebException e)
                {
                    Trace.WriteLine("cannot download feed URI '" + feedURI + "': " + e.ToString());
                    throw;
                }
                catch (XmlException e)
                {
                    Trace.WriteLine("cannot parse feed URI '" + feedURI + "': " + e.ToString());
                    throw;
                }

                Trace.WriteLine("downloaded successfully feed URI '" + feedURI + "'");
            }
        }

        public void RefreshFeedEvent(Object sender, EventArgs eventArgs)
        {
            try
            {
                RefreshFeed();
            }
            catch
            {
                XmlDocument xmlDoc = new XmlDocument();
                xmlDoc.LoadXml(Properties.Resources.DefaultRSSText);
                LoadFromNode(xmlDoc);
            }
        }

        public void Dispose()
        {
            DisposeChannels();
        }

        private void DisposeChannels()
        {
            if (channels != null)
            {
                foreach (RssChannel channel in channels)
                {
                    channel.Dispose();
                }
            }
        }
    }
}