// --------------------------------------------------------------------------
//
// Media Feed Screen Saver
// http://blog.debin.net/Media-feed-screen-saver/
// damien@debin.net
//
// --------------------------------------------------------------------------
//
// Copyright (c) 2006 Damien Debin.
// All rights reserved.
// 
// This file is part of Media Feed Screen Saver.
// 
// Media Feed Screen Saver is free software; you can redistribute it
// and/or modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
// 
// Foobar is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Foobar; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
// --------------------------------------------------------------------------

using System;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Net;
using System.Net.Cache;
using System.Threading;
using System.Xml;
using MediaFeedScreenSaver.ResizingFilters;

namespace MediaFeedScreenSaver.Rss
{
    /// <summary>
    /// Representation of an Item element in an RSS 2.0 XML document.
    /// Zero or more RssItems are contained in an RssChannel.
    /// </summary>
    public class RssItem : IDisposable
    {
        private readonly string title;
        private readonly string description;
        private readonly string link;
        private readonly Size imageSize;
        private string enclosureImageURI;

        private Image bgImage = null;
        private bool enable = false;

        public string Title { get { return title; } }
        public string Description { get { return description; } }
        public string Link { get { return link; } }
        public string EnclosureImageURI { get { return enclosureImageURI; } }
        public bool Enable { get { return enable; } }
        public Image BgImage { get { return bgImage; } }

        public static object concurrentDownloadLock = new object();

        /// <summary>
        /// Build an RSSItem from an XmlNode representing an Item element inside an RSS 2.0 XML document.
        /// </summary>
        /// <param name="itemNode"></param>
        internal RssItem(XmlNode itemNode, Size requestedImageSize)
        {
            XmlNode selected;

            selected = itemNode.SelectSingleNode("title");
            if (selected != null)
                title = selected.InnerText;

            selected = itemNode.SelectSingleNode("description");
            if (selected != null)
                description = selected.InnerText;

            selected = itemNode.SelectSingleNode("link");
            if (selected != null)
                link = selected.InnerText;

            selected = itemNode.SelectSingleNode("enclosure");
            if (selected != null)
            {
                enclosureImageURI = selected.Attributes["url"].InnerText;
            }

            XmlNamespaceManager nsmgr = new XmlNamespaceManager(itemNode.OwnerDocument.NameTable);
            nsmgr.AddNamespace("media", itemNode.GetNamespaceOfPrefix("media"));

            selected = itemNode.SelectSingleNode("media:content", nsmgr);
            if (selected != null)
            {
                enclosureImageURI = selected.Attributes["url"].InnerText; //override enclosureImage if exists
            }

            //for Apple Photocast
            if (String.IsNullOrEmpty(enclosureImageURI) && !String.IsNullOrEmpty(itemNode.GetNamespaceOfPrefix("apple-wallpapers")))
            {
                nsmgr.AddNamespace("apple-wallpapers", itemNode.GetNamespaceOfPrefix("apple-wallpapers"));
                selected = itemNode.SelectSingleNode("apple-wallpapers:image", nsmgr);
                if (selected != null)
                {
                    enclosureImageURI = selected.InnerText; //Try to find this instead?
                }
            }

            //for http://www.pheed.com/pheed/ extension
            if (String.IsNullOrEmpty(enclosureImageURI) && !String.IsNullOrEmpty(itemNode.GetNamespaceOfPrefix("photo")))
            {
                nsmgr.AddNamespace("photo", itemNode.GetNamespaceOfPrefix("photo"));
                selected = itemNode.SelectSingleNode("photo:imgsrc", nsmgr);
                if (selected != null)
                {
                    enclosureImageURI = selected.InnerText; //Try to find this instead?
                }
            }

            if (!String.IsNullOrEmpty(enclosureImageURI) && (requestedImageSize != Size.Empty))
            {
                imageSize = requestedImageSize;
                Thread t = new Thread(new ThreadStart(RefreshImage));
                t.Start();
            }
        }

        /// <summary>
        /// Dispose all disposable fields
        /// </summary>
        public void Dispose()
        {
            if (bgImage != null)
            {
                bgImage.Dispose();
            }
        }
        
        private Bitmap ResizeImage(Image i, int nWidth, int nHeight)
        {
            int imageW = i.Width;
            int imageH = i.Height;
            float RW = (float)nWidth / imageW;
            float RH = (float)nHeight / imageH;
            float R = Math.Min(RW, RH);
            int sizeW = Convert.ToInt32(imageW * R);
            int sizeH = Convert.ToInt32(imageH * R);

            if (Properties.Settings.Default.HighQualityResizing)
            {
                using (Bitmap b = new Bitmap(i))
                {
                    ResamplingService resampling = new ResamplingService(b, 11);
                    resampling.Resample(sizeW, sizeH);
                    return resampling.Output;
                }
            }
            else
            {
                Bitmap result = new Bitmap(sizeW, sizeH);
                using (Graphics g = Graphics.FromImage(result))
                {
                    g.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.Bilinear;
                    g.DrawImage(i, 0, 0, sizeW, sizeH);
                }
                return result;
            }

            /*ResamplingService resampling = null;
            static int ii = 0;
            byte j;
            for (j = 0; j < 13; j++)
            {
                resampling = new ResamplingService(new Bitmap(i), j);
                resampling.Resample(sizeW, sizeH);
                resampling.Output.Save(@"c:\" + ii.ToString() + "_" + j.ToString() + ".png", System.Drawing.Imaging.ImageFormat.Png);
            }

            Bitmap result = new Bitmap(sizeW, sizeH);
            using (Graphics g = Graphics.FromImage(result))
            {
                g.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.Bilinear;
                g.DrawImage(i, 0, 0, sizeW, sizeH);
                result.Save(@"c:\" + ii.ToString() + "_" + j.ToString() + ".png", System.Drawing.Imaging.ImageFormat.Png);
            }
            ii++;*/
        }

        private void RefreshImage()
        {
            lock (concurrentDownloadLock) //to prevent multiple concurrent downloads
            {
                WebClient webClient = new WebClient();
                RequestCachePolicy policy = new RequestCachePolicy(RequestCacheLevel.CacheIfAvailable);
                webClient.CachePolicy = policy;

                try
                {
                    Image image;
                    using (Stream webStream = webClient.OpenRead(enclosureImageURI))
                    {
                        image = Image.FromStream(webStream, true, true);
                    }

                    if (bgImage != null) bgImage.Dispose();
                    bgImage = ResizeImage(image, imageSize.Width, imageSize.Height);
                    image.Dispose();

                    enable = true;
                }
                catch (WebException e)
                {
                    Trace.WriteLine("cannot download URI '" + EnclosureImageURI + "': " + e.ToString());
                    enclosureImageURI = null;
                    return;
                }
                catch (OutOfMemoryException e)
                {
                    Trace.WriteLine("cannot load image URI '" + EnclosureImageURI + "' (too big): " + e.ToString());
                    enclosureImageURI = null;
                    return;
                }
                catch (ArgumentException e)
                {
                    Trace.WriteLine("cannot decode image URI '" + EnclosureImageURI + "': " + e.ToString());
                    enclosureImageURI = null;
                    return;
                }

                Trace.WriteLine("downloaded successfully URI '" + EnclosureImageURI + "'");
            }
        }
    }
}