// --------------------------------------------------------------------------
//
// Media Feed Screen Saver
// http://blog.debin.net/Media-feed-screen-saver/
// damien@debin.net
//
// --------------------------------------------------------------------------
//
// Copyright (c) 2006 Damien Debin.
// All rights reserved.
// 
// This file is part of Media Feed Screen Saver.
// 
// Media Feed Screen Saver is free software; you can redistribute it
// and/or modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
// 
// Foobar is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Foobar; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
// --------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Net;
using ST = System.Timers;
using System.Threading;
using System.Windows.Forms;
using SWF = System.Windows.Forms;
using MediaFeedScreenSaver.Rss;

namespace MediaFeedScreenSaver
{
    partial class ScreenSaverForm : Form
    {
        // The RssFeed to display pictures from
        private RssFeed rssFeed;
        private RssChannel displayedChannel;
        private int currentFeedIndex;

        // Objects for displaying RSS title
        private ItemDescriptionView rssDescriptionView;

        // Keep track of whether the screensaver has become active.
        private bool isActive = false;

        // Keep track of the location of the mouse
        private Point mouseLocation;

        private bool delayedInitDone = false;

        private ST.Timer refreshFeedTimer;
        private SWF.Timer refreshScreenTimer;

        public ScreenSaverForm(Screen display)
        {
            InitializeComponent();
#if !DEBUG
            SetupScreenSaver(display);
#endif
        }

        // this init must be delayed until first call of OnPaint
        // otherwise the size (value used in this init) of the form is incorrect
        private void DelayedInit()
        {
            LoadRssFeed();

            // Initialize the ItemDescriptionView to display the description of the RssItem.
            rssDescriptionView = new ItemDescriptionView();

            InitializeRssDescriptionView();
        }

        /// <summary>
        /// Set up the main form as a full screen screensaver.
        /// </summary>
        private void SetupScreenSaver(Screen display)
        {
            // Use double buffering to improve drawing performance
            SetStyle(ControlStyles.OptimizedDoubleBuffer | ControlStyles.UserPaint | ControlStyles.AllPaintingInWmPaint, true);
            // Capture the mouse
            Capture = true;

            // Set the application to full screen mode and hide the mouse
            Cursor.Hide();
            Bounds = display.Bounds;
            WindowState = FormWindowState.Maximized;
            ShowInTaskbar = false;
            DoubleBuffered = true;
        }

        private void LoadRssFeed()
        {
            // Try to get it from the users settings
            rssFeed = new RssFeed(null, Properties.Settings.Default.RssFeedUri, this.Size);
            displayedChannel = null;
            currentFeedIndex = 0;

            // Load RSS feed in background
            System.Threading.Thread t = new Thread(new ThreadStart(this.DelayedRssFeedLoad));
            t.Start();

            // Start feed refresh timer
            refreshFeedTimer = new ST.Timer();
            refreshFeedTimer.Elapsed += new ST.ElapsedEventHandler(rssFeed.RefreshFeedEvent);
            refreshFeedTimer.Interval = Properties.Settings.Default.FeedRefreshInterval * 1000;
            refreshFeedTimer.Enabled = true;
            refreshFeedTimer.Start();
        }

        private void DelayedRssFeedLoad()
        {
            rssFeed.RefreshFeedEvent(null, null);
            nextPicture();
        }

        /// <summary>
        /// Initialize display properties of the rssDescriptionView.
        /// </summary>
        private void InitializeRssDescriptionView()
        {
            int fontSize = Height / 35;
            rssDescriptionView.DisplayItem = null;
            rssDescriptionView.ForeColor = Properties.Settings.Default.TextColor;
            rssDescriptionView.TitleFont = new Font("Microsoft Sans Serif", fontSize, GraphicsUnit.Pixel);
            rssDescriptionView.Location = new Point(Width, Height);
            rssDescriptionView.Size = new Size(Width, fontSize * 4);

            refreshScreenTimer = new SWF.Timer();
            refreshScreenTimer.Interval = rssDescriptionView.fadeTimerInterval;
            refreshScreenTimer.Tick += new EventHandler(FadeTimer_Tick);
            refreshScreenTimer.Enabled = true;
            refreshScreenTimer.Start();
        }

        private void ScreenSaverForm_MouseMove(object sender, MouseEventArgs e)
        {
            // Set IsActive and MouseLocation only the first time this event is called.
            if (!isActive)
            {
                mouseLocation = MousePosition;
                isActive = true;
            }
            else
            {
                // If the mouse has moved significantly since first call, close.
                if ((Math.Abs(MousePosition.X - mouseLocation.X) > 10) ||
                    (Math.Abs(MousePosition.Y - mouseLocation.Y) > 10))
                {
                    Close();
                }
            }
        }

        private void ScreenSaverForm_KeyDown(object sender, KeyEventArgs e)
        {
            Close();
        }

        private void ScreenSaverForm_MouseDown(object sender, MouseEventArgs e)
        {
            Close();
        }

        protected override void OnPaintBackground(PaintEventArgs e)
        {
            try
            {
                if ((rssDescriptionView == null) || (rssDescriptionView.DisplayItem == null) || (rssDescriptionView.DisplayItem.BgImage == null)) return;

                int sizeW = rssDescriptionView.DisplayItem.BgImage.Width;
                int sizeH = rssDescriptionView.DisplayItem.BgImage.Height;
                int posX = (Size.Width - sizeW) / 2;
                int posY = (Size.Height - sizeH) / 2;
                Point pos = new Point(posX, posY);

                e.Graphics.CompositingQuality = System.Drawing.Drawing2D.CompositingQuality.AssumeLinear; //HighQuality;
                using (Brush bgBrush = new SolidBrush(Properties.Settings.Default.BkgdColor))
                {
                    e.Graphics.FillRectangle(bgBrush, Location.X, Location.Y, Size.Width, Size.Height);
                }
                e.Graphics.DrawImageUnscaled(rssDescriptionView.DisplayItem.BgImage, pos);
            }
            catch
            { }
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            if (!delayedInitDone)
            {
                delayedInitDone = true;
                DelayedInit();
            }

            rssDescriptionView.Paint(e);
        }

        private void FadeTimer_Tick(object sender, EventArgs e)
        {
            if (rssDescriptionView.titleFadingAnimate())
            {
                nextPicture();
            }
            Refresh();
        }

        private void nextPicture()
        {
            // Update displayed channel in case XML feed has been refreshed
            displayedChannel = rssFeed.MainChannel;

            if (displayedChannel == null) return;

            rssDescriptionView.AllPicturesStillNotDownloaded = false;
            for (int i = 0; i < displayedChannel.Items.Count; i++)
            {
                if ((displayedChannel.Items[i].EnclosureImageURI != null) && (!displayedChannel.Items[i].Enable))
                {
                    rssDescriptionView.AllPicturesStillNotDownloaded = true;
                    break;
                }
            }

            // Change the displayed item (background image) to the next item.
            for (int i = 0; i < displayedChannel.Items.Count; i++)
            {
                currentFeedIndex = (currentFeedIndex + 1) % displayedChannel.Items.Count;
                rssDescriptionView.DisplayItem = displayedChannel.Items[currentFeedIndex];
                if (rssDescriptionView.DisplayItem.Enable) break; // we found one
            }
        }
    }
}
